""" Copyright 2024-2025 LEDR Technologies Inc.
This file is part of the Orchestra library, which helps developer use our Orchestra technology which is based on AvesTerra, owned and developped by Georgetown University, under license agreement with LEDR Technologies Inc.
The Orchestra library is a free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
The Orchestra library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
You should have received a copy of the GNU Lesser General Public License along with the Orchestra library. If not, see <https://www.gnu.org/licenses/>.
If you have any questions, feedback or issues about the Orchestra library, you can contact us at support@ledr.io.
"""

from __future__ import annotations
from avesterra.avial import AvAuthorization, AvEntity, NULL_ENTITY
from avesterra.taxonomy import AvContext, AvClass, AvCategory
from oop.entity import Entity
import avesterra.registries as registries
from oop.record import Record


class Registry(Entity):
    def __init__(
        self,
        authorization: AvAuthorization,
        registry: AvEntity = NULL_ENTITY,
        name: str = "",
        category: AvCategory = AvCategory.REGISTRY,
        outlet: AvEntity = AvEntity,
        auto_dep_res: bool = False,
    ):
        super().__init__(
            authorization=authorization,
            entity=registry,
            name=f"{name} Registry" if "Registry" not in name else f"{name}",
            key=(f"{name} Registry" if "Registry" not in name else f"{name}")
            .lower()
            .replace(" ", "_"),
            context=AvContext.AVESTERRA,
            klass=AvClass.REGISTRY,
            category=category,
            outlet=outlet,
            auto_dep_res=auto_dep_res,
        )

    def get(self, key: str = "", index: int = 0) -> Record:
        clean_key: str = key

        if key == "":
            clean_key = registries.registry_key(
                registry=self.entity, index=index, authorization=self.authorization
            )

        # Return Record object
        record: Record = Record(
            parent=self,
            registry=self.entity,
            key=clean_key,
            entity=registries.lookup_registry(
                registry=self.entity, key=clean_key, authorization=self.authorization
            ),
            authorization=self.authorization,
            auto_dep_res=self.auto_dep_res,
        )

        return record

    def register(self, key: str, name: str, entity: AvEntity) -> Record:
        # Return Record object
        record: Record = Record(
            parent=self,
            registry=self.entity,
            key=key,
            entity=entity,
            authorization=self.authorization,
            auto_dep_res=self.auto_dep_res,
        )

        registries.register_entity(
            registry=self.entity,
            key=key,
            name=name,
            entity=entity,
            authorization=self.authorization,
        )

        return record

    def deregister(self, key: str) -> Registry:
        registries.deregister_entity(
            registry=self.entity, key=key, authorization=self.authorization
        )
        return self

    def purge(self) -> Registry:
        registries.purge_registry(
            registry=self.entity, authorization=self.authorization
        )
        return self

    def delete(self):
        registries.delete_registry(
            registry=self.entity, authorization=self.authorization
        )
